<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( !class_exists( 'WooCommerce_Customer_Manager_Settings' ) ) {

	class WooCommerce_Customer_Manager_Settings {
		public function __construct() {
			$this->settings = get_option( 'wpo_wccm_settings' );
			add_action( 'admin_menu', array( $this, 'add_menu' ), 100 ); // not really :o)
			add_action( 'admin_init', array( $this, 'init_settings' ) ); // Registers settings
			add_filter( 'option_page_capability_wpo_wccm_settings', array( &$this, 'settings_capabilities' ) );
		}

		public function add_menu() {
			// adding to NULL because we're linking from the customers page (saves unnecessary menu clutter)
			add_submenu_page(
				NULL, //'woocommerce',
				'WooCommerce Customer Manager Settings',
				'WooCommerce Customer Manager Settings',
				'manage_woocommerce',
				'wpo_wccm_options_page',
				array( $this, 'settings_page' )
			);
		}

		/**
		 * Set capability for settings page
		 */
		public function settings_capabilities() {
			return 'manage_woocommerce';
		}

		public function settings_page() {
			?>
			<div class="wrap">
				<div class="icon32" id="icon-options-general"><br /></div>
				<h2><?php _e( 'WooCommerce Customer Manager Settings', 'wpo_wccm' ); ?></h2>
				<form method="post" action="options.php">
					<?php
						$update_url = add_query_arg( 'update_wc_customers', 'true' );
						$import = __( 'Import customers from existing orders', 'wpo_wccm' );
						$class = get_option( 'wpo_wccm_last_sync' ) ? 'button' : 'button button-primary';
						printf('<a href="%s" class="%s">%s</a>', $update_url, $class, $import);
						if ( $last_sync = get_option( 'wpo_wccm_last_sync' ) ) {
							printf('<p class="description">%s %s</p>', __( 'Last import:', 'wpo_wccm' ), date_i18n( get_option( 'date_format' ) .' ' . get_option( 'time_format' ), strtotime( $last_sync ) ) );
						}

						settings_fields( 'wpo_wccm_settings' );
						do_settings_sections( 'wpo_wccm_settings' );

	
						submit_button();
					?>
				</form>
					
			</div>
			<?php
		}		

		/**
		 * Number settings.
		 */
		public function init_settings() {
			global $woocommerce;
	
			$option = 'wpo_wccm_settings';
		
			// Create option in wp_options.
			if ( false === get_option( $option ) ) {
				$this->default_settings( $option );
			}
		
			// Section.
			add_settings_section(
				'general_settings',
				__( 'General settings', 'wpo_wccm' ),
				array( &$this, 'section_options_callback' ),
				$option
			);
	
			add_settings_field(
				'next_number',
				__( 'Next customer number', 'wpo_wccm' ),
				array( &$this, 'text_element_callback' ),
				$option,
				'general_settings',
				array(
					'menu'			=> $option,
					'id'			=> 'next_number',
					'size'			=> '4',
					'default'		=> 1001,
					'description'	=> __( 'This number will be assigned to the next customer that places his/her first order or the first number that is assigned when an import starts.', 'wpo_wccm' ),
				)
			);

			add_settings_field(
				'number_prefix',
				__( 'Number prefix', 'wpo_wccm' ),
				array( &$this, 'text_element_callback' ),
				$option,
				'general_settings',
				array(
					'menu'			=> $option,
					'id'			=> 'number_prefix',
					'size'			=> '40',
				)
			);
		
			add_settings_field(
				'sync_orders',
				__( 'Auto sync order data', 'wpo_wccm' ),
				array( &$this, 'checkbox_element_callback' ),
				$option,
				'general_settings',
				array(
					'menu'			=> $option,
					'id'			=> 'sync_orders',
					'description'	=> __( 'Update customer information when a new order is placed or if an existing order is changed.', 'wpo_wccm' ),
				)
			);

			add_settings_field(
				'connect_old_orders',
				__( 'Auto-connect orders', 'wpo_wccm' ),
				array( &$this, 'checkbox_element_callback' ),
				$option,
				'general_settings',
				array(
					'menu'			=> $option,
					'id'			=> 'connect_old_orders',
					'description'	=> __( 'Automatically connect old orders when a new customer is created.', 'wpo_wccm' ),
				)
			);

			add_settings_field(
				'show_card',
				__( 'Show customer card on order page', 'wpo_wccm' ),
				array( &$this, 'checkbox_element_callback' ),
				$option,
				'general_settings',
				array(
					'menu'			=> $option,
					'id'			=> 'show_card',
				)
			);

			if ( class_exists( 'WooCommerce_PDF_Invoices' ) ) {
				$pdf_docs = array(
					'invoice'		=> __( 'Invoice', 'wpo_wcpdf' ),
					'packing-slip'	=> __( 'Packing Slip', 'wpo_wcpdf' ),
				);

				if ( class_exists( 'WooCommerce_PDF_IPS_Pro' ) ) {
					$pdf_pro_docs = array(
						'proforma'		=> __( 'Proforma Invoice', 'wpo_wcpdf_pro' ),
						'credit-note'	=> __( 'Credit Note', 'wpo_wcpdf_pro' ),
					);

					$pdf_docs = $pdf_docs + $pdf_pro_docs;
				}

				add_settings_field(
					'wcpdf_show',
					__( 'Add customer number to PDF', 'wpo_wccm' ),
					array( &$this, 'multiple_checkbox_element_callback' ),
					$option,
					'general_settings',
					array(
						'menu'			=> $option,
						'id'			=> 'wcpdf_show',
						'options'		=> $pdf_docs,
					)
				);
			}

			add_settings_field(
				'number_wc_emails',
				__( 'Add customer number to WooCommerce emails', 'wpo_wccm' ),
				array( &$this, 'checkbox_element_callback' ),
				$option,
				'general_settings',
				array(
					'menu'			=> $option,
					'id'			=> 'number_wc_emails',
				)
			);

			add_settings_field(
				'number_my_account',
				__( 'Show customer number on customer My Account page', 'wpo_wccm' ),
				array( &$this, 'checkbox_element_callback' ),
				$option,
				'general_settings',
				array(
					'menu'			=> $option,
					'id'			=> 'number_my_account',
				)
			);

			add_settings_field(
				'custom_fields',
				__( 'Customer card fields', 'wpo_wccm' ),
				array( &$this, 'custom_fields_callback' ),
				$option,
				'general_settings',
				array(
					'menu'			=> $option,
					'id'			=> 'custom_fields',
					'description'	=> __( 'Add custom fields that you want to use on the customer card. Changing the field name is optional.', 'wpo_wccm' ),
				)
			);

			// Register settings.
			register_setting( $option, $option, array( &$this, 'validate_options' ) );
		}

		/**
		 * Set default settings.
		 */
		public function default_settings( $option ) {
			// assuming wpo_wccm_settings for now
			$default = array(
				'show_card'				=> 1,
				'sync_orders'			=> 1,
				'connect_old_orders'	=> 1
			);

			if ( false === get_option( $option ) ) {
				add_option( $option, $default );
			} else {
				update_option( $option, $default );
			}
		}

		/**
		 * Text field callback.
		 */
		public function text_element_callback( $args ) {
			$menu = $args['menu'];
			$id = $args['id'];
			$size = isset( $args['size'] ) ? $args['size'] : '25';
		
			$options = get_option( $menu );
		
			if ( isset( $options[$id] ) ) {
				$current = $options[$id];
			} else {
				$current = isset( $args['default'] ) ? $args['default'] : '';
			}
		
			printf( '<input type="text" id="%1$s" name="%2$s[%1$s]" value="%3$s" size="%4$s"/>', $id, $menu, $current, $size );
		
			// Displays option description.
			if ( isset( $args['description'] ) ) {
				printf( '<p class="description" style="width: 400px;">%s</p>', $args['description'] );
			}
		}
	
		/**
		 * Checkbox field callback.
		 */
		public function checkbox_element_callback( $args ) {
			$menu = $args['menu'];
			$id = $args['id'];
			$value = isset( $args['value'] ) ? $args['value'] : 1;
		
			$options = get_option( $menu );
		
			if ( isset( $options[$id] ) ) {
				$current = $options[$id];
			} else {
				$current = isset( $args['default'] ) ? $args['default'] : '';
			}
		
			printf( '<input type="checkbox" id="%1$s" name="%2$s[%1$s]" value="%3$s"%4$s />', $id, $menu, $value, checked( $value, $current, false ) );
		
			// Displays option description.
			if ( isset( $args['description'] ) ) {
				printf( '<p class="description" style="width: 400px;">%s</p>', $args['description'] );
			}
		}

		/**
		 * Multiple Checkbox field callback.
		 *
		 * @param  array $args Field arguments.
		 *
		 * @return string	  Checkbox field.
		 */
		public function multiple_checkbox_element_callback( $args ) {
			$menu = $args['menu'];
			$id = $args['id'];
		
			$options = get_option( $menu );
		
		
			foreach ( $args['options'] as $key => $label ) {
				$current = ( isset( $options[$id][$key] ) ) ? $options[$id][$key] : '';
				printf( '<input type="checkbox" id="%1$s[%2$s][%3$s]" name="%1$s[%2$s][%3$s]" value="1"%4$s /> %5$s<br/>', $menu, $id, $key, checked( 1, $current, false ), $label );
			}

			// Displays option description.
			if ( isset( $args['description'] ) ) {
				printf( '<p class="description">%s</p>', $args['description'] );
			}
		}

		/**
		 * Custom fields callback.
		 */
		public function custom_fields_callback( $args ) {
			$menu = $args['menu'];
			$id = $args['id'];
			$value = isset( $args['value'] ) ? $args['value'] : 1;
		
			$options = get_option( $menu );
		
			if ( isset( $options[$id] ) ) {
				$current_fields = $options[$id];
			}
			
			// Displays option description.
			if ( isset( $args['description'] ) ) {
				printf( '<p class="description">%s</p>', $args['description'] );
			}
			?>
			<table id="custom_fields">
				<thead>
					<tr id="title_row">
						<td><?php _e( 'Type', 'wpo_wccm' ); ?></td>
						<td><?php _e( 'Label', 'wpo_wccm' ); ?></td>
						<td><?php _e( 'Field name (for database)', 'wpo_wccm' ); ?></td>
						<td></td>
					</tr>
					</thead>
				<tbody>
					<?php
					if ( !isset($current_fields) || empty($current_fields) ) {
						$current_fields = array(
							array(
								'type'	=> '',
								'label'	=> '',
								'field'	=> '',
							),
						);
					}
					foreach ($current_fields as $key => $custom_field) {
					?>
					<tr data-id="<?php echo $key; ?>">
						<td>
							<?php
							printf( '<select id="%1$s[%2$s][%3$s][type]" class="%2$s_type" name="%1$s[%2$s][%3$s][type]">', $menu, $id, $key );
							$select_options = array(
								'checkbox'	=> __( 'Checkbox', 'wpo_wccm' ),
								'text'		=> __( 'Textbox', 'wpo_wccm' ),
								// 'dropdown'	=> __( 'Dropdown', 'wpo_wccm' ),
							);

							foreach ($select_options as $value => $label) {
								printf( '<option value="%s"%s>%s</option>', $value, selected( $custom_field['type'], $value, false ), $label  );
							}
							?>
							</select>
						</td>
						<td>
							<?php
							printf( '<input type="text" id="%1$s[%2$s][%3$s][label]" class="%2$s_label" name="%1$s[%2$s][%3$s][label]" value="%4$s">', $menu, $id, $key, $custom_field['label'] );
							?>
						</td>
						<td>
							<?php
							printf( '<input type="text" id="%1$s[%2$s][%3$s][field]" class="%2$s_field" name="%1$s[%2$s][%3$s][field]" value="%4$s">', $menu, $id, $key, $custom_field['field'] );
							?>
						</td>
						<td><a class="delete_custom_box"><span class="dashicons dashicons-dismiss"></span></a></td>
					</tr>
					<?php
					} //endforeach
					?>
				</tbody>
			</table>

			<a class="button add_new_custom_box"><?php _e( 'Add custom field', 'wpo_wccm' ); ?></a>
			<script  type="text/javascript">
				jQuery( function ( $ ) {
					$('.add_new_custom_box').click(function() {
						var last_row = $("#custom_fields tr:last");

						var last_key = $(last_row).attr("data-id");						
						last_key++;

						// clone template row
						var new_row = $(last_row).clone().attr('data-id', last_key);
						
						// add row counter id to name & id
						var id_base = 'wpo_wccm_settings[custom_fields]'+'['+last_key+']';
						new_row.find('select').attr('id', id_base+'[type]').attr('name', id_base+'[type]');
						new_row.find('input').first().attr('id', id_base+'[label]').attr('name', id_base+'[label]');
						new_row.find('input').last().attr('id', id_base+'[field]').attr('name', id_base+'[field]');
						
						// clear values
						new_row.find('input:text').val('');

						// finally, add the new row
						new_row.insertAfter( last_row );

					});
					$('#custom_fields').on( 'click', '.delete_custom_box', function() {
						var num_rows = $('#custom_fields tbody tr').length;
						if (num_rows > 1) {
							$(this).closest('tr').remove();
						} else {
							$(this).closest('tr').find('input:text').val('');
						}
					});
					$('#custom_fields').on( 'focusout', '.custom_fields_label', function() {
						var label = $(this).val();
						$(this).parent().next().find('input').val(convertToSlug(label));
					});

				});

				function convertToSlug(Text) {
					return Text
						.toLowerCase()
						.replace(/ /g,'_')
						.replace(/[-]+/g, '_')
						.replace(/[^\w-]+/g,'');
				}
			</script>
			<?php

		}

		/**
		 * Section null callback.
		 */
		public function section_options_callback() {
			// silence is golden
		}

		/**
		 * Validate options.
		 *
		 * @param  array $input options to valid.
		 *
		 * @return array		validated options.
		 */
		public function validate_options( $input ) {
			// No validation required for now
			// Return the array processing any additional functions filtered by this action.
			return apply_filters( 'wpo_wccm_validate_input', $input );
		}

	} // end class
} // end class_exists