<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( !class_exists( 'WooCommerce_Customer_Manager_Writepanels' ) ) {

	class WooCommerce_Customer_Manager_Writepanels {
		public function __construct() {
			$this->settings = get_option( 'wpo_wccm_settings' );

			// add_action( 'admin_menu', array( $this, 'add_menu' ), 100 );
			add_action( 'admin_enqueue_scripts', array( $this, 'load_scripts_styles' ) );

			add_filter( 'manage_edit-wc_customers_columns', array( $this, 'customers_colums' ) );
			add_filter( 'manage_wc_customers_posts_custom_column', array( $this, 'customers_colum_data' ) );
			add_filter( 'manage_edit-wc_customers_sortable_columns', array( $this, 'numbers_sortable' ) );

			add_filter( 'views_edit-wc_customers', array( $this, 'settings_page_link' ) );
			add_filter( 'views_edit-wc_customers', array( $this, 'export_link' ) );
			add_action( 'admin_footer-edit.php', array( $this, 'move_buttons' ) ); // no proper hooks so we're injecting with jQUery

			add_action( 'admin_menu', array( $this, 'remove_submit_box' ) );
			add_action( 'admin_init', array( $this, 'add_meta_boxes' ) );
			add_filter( 'post_updated_messages', array( $this, 'post_updated_messages' ) );
			add_action( 'woocommerce_admin_order_actions_end', array( $this, 'add_order_actions' ) );

			add_action( 'admin_notices', array( $this, 'sync_notice' ) );

			if ( ! empty( $_GET['wc_customers_imported'] ) ) {
				add_action( 'admin_notices', array( $this, 'import_done' ) );
			}

			add_filter( 'admin_comment_types_dropdown', array( $this, 'filter_by_customer_note' ) );

			add_filter( 'woocommerce_email_customer_details_fields', array( $this, 'email_customer_number' ), 10, 3 );
			add_action( 'woocommerce_after_my_account', array( $this, 'my_account_customer_number' ) );
			add_action( 'show_user_profile', array( $this, 'user_profile_customer_number' ), 5, 1 );
		}

		public function sync_notice () {
			if ( !( $last_sync = get_option( 'wpo_wccm_last_sync' ) ) && !( get_option( 'wpo_wccm_ignore_import' ) ) ) {
				$notice = __( "It looks like this is the first time you're running WooCommerce Customer Manager.", 'wpo_wccm');
				$import = __( 'Import customers from existing orders', 'wpo_wccm' );
				$dismiss = __( 'Ignore', 'wpo_wccm' );
				$warning = __( 'note that importing may take a while for shops with a lot of existing orders', 'wpo_wccm' );

				printf( '<div class="updated fade"><p>%s<br/>(%s)<br/><a href="%s" class="button">%s</a> <a href="%s" class="button">%s</a></p></div>', $notice, $warning, add_query_arg( 'update_wc_customers', 'true' ), $import, add_query_arg( 'ignore_wc_customers_import', 'true' ), $dismiss );
			}
			
		}

		public function import_done () {
			if (isset($_GET['import_count'])) {
				$count = $_GET['import_count'];
			} else {
				$count = 0;
			}

			$notice = sprintf( __( 'Imported/updated %d customers', 'wpo_wccm' ), $count);
			printf( '<div class="updated fade"><p>%s</p></div>', $notice);
		}

		public function add_menu() {
			add_submenu_page(
				'woocommerce',
				'Customers',
				'Customers',
				'manage_woocommerce',
				'edit.php?post_type=wc_customers'
			);
		}

		public function settings_page_link( $views ) {
			$url = admin_url( 'admin.php?page=wpo_wccm_options_page' );
			$title = __( 'Configure settings', 'wpo_wccm' );
			$button = sprintf('<a href="%s" id="wc_customers_settings" class="add-new-h2"  title="%s" style="margin:5px">%s</a>', $url, esc_attr( $title ), $title );
			$views['customer-settings'] = $button;
			return $views;
		}

		public function export_link( $views ) {
			$url = wp_nonce_url( admin_url( 'admin-ajax.php?action=wpo_wcccm_export' ), 'wpo_wcccm_export' );
			$title = __( 'Export to CSV', 'wpo_wccm' );
			$button = sprintf('<a href="%s" id="wc_customers_export" class="button"  title="%s">%s</a>', $url, esc_attr( $title ), $title );
			$views['customer-export'] = $button;
			return $views;
		}

		public function move_buttons () {
			global $post_type, $wp_version;
			if ($post_type != 'wc_customers') return;

			// Post type heading: H1 for WP4.3+, H2 for anything older
			$h = version_compare( $wp_version, '4.3', '<' ) ? 'h2' : 'h1';
			?>
			<script type="text/javascript">
				jQuery(document).ready( function($) 
				{
					if($(".wrap <?php echo $h; ?>:first span").length) {
						// settings link before "Search results for ..."
						$('#wc_customers_settings').insertBefore('.wrap <?php echo $h; ?>:first span');
					} else {
						$('#wc_customers_settings').appendTo('.wrap <?php echo $h; ?>:first');
					}
					$('li.customer-settings').remove();
					
					$('#wc_customers_export').insertAfter('#post-query-submit');
					$('li.customer-export').remove();
				});
			</script>
			<?php 
		}

		/**
		 * Styles for Customers Page
		 */
		public function load_scripts_styles ( $hook ) {
			// die($hook);
			global $wp_version, $post_type, $post;

			// use WP 4.1 dashicons on older WP versions
			if ( version_compare( $wp_version, '4.1', '<' ) ) {
				wp_enqueue_style( 'wpo-wccm-dashicons', plugins_url( 'css/dashicons.css' , dirname(__FILE__) ) );
			}

			if ( !in_array( $post_type, array( 'wc_customers', 'shop_order' ) ) && $hook != 'admin_page_wpo_wccm_options_page' ) return;
			
			wp_enqueue_script(
				'wpo-wccm',
				plugins_url( 'js/wpo-wccm.js' , dirname(__FILE__) ),
				array('jquery', 'jquery-blockui'),
				'1.2.5'
			);
			wp_enqueue_style(
				'wpo-wccm',
				plugins_url( 'css/admin-styles.css' , dirname(__FILE__) ),
				'', //deps
				'1.2.5'
			);

			$localize_params = array(
				'plugin_url'                    => plugin_dir_url( dirname(__FILE__) ),
				'ajax_url'                      => admin_url('admin-ajax.php'),
				'add_customer_note_nonce'       => wp_create_nonce("add-customer-note"),
				'delete_customer_note_nonce'    => wp_create_nonce("delete-customer-note"),
				'save_customer_number_nonce'    => wp_create_nonce("save-customer-number"),
				'post_id'                       => isset( $post->ID ) ? $post->ID : '',
			);

			wp_localize_script( 'wpo-wccm', 'wpo_wccm', $localize_params );

		}

		public function customers_colums ( $existing_columns ) {
			$columns = array(
				'cb'						=> '<input type="checkbox" />',
				'customer_number'			=> __( 'Number', 'wpo_wccm' ),
				'customer_name'				=> __( 'Name', 'wpo_wccm' ),
				'customer_billing_address'	=> __( 'Billing Address', 'wpo_wccm' ),
				'customer_shipping_address'	=> __( 'Shipping Address', 'wpo_wccm' ),
				'customer_orders'			=> __( 'Orders', 'wpo_wccm' ),
				'customer_last_order'		=> __( 'Last Order', 'wpo_wccm' )
			);

			return $columns;
		}

		public function numbers_sortable ( $columns ) {
			$columns['customer_number'] = 'customer_number';
			$columns['customer_orders'] = 'order_count';
			$columns['customer_last_order'] = 'last_order';
			return $columns;
		}

		public function customers_colum_data ( $column ) {
			global $post, $wpdb, $wpo_wccm;
			$customer = $post;

			switch ( $column ) {

				case 'customer_number' :
					$customer_number = $wpo_wccm->functions->get_customer_number( $customer->ID );
					printf('<a href="post.php?post=%1$s&action=edit">%2$s</a>', $customer->ID, $customer_number );
					echo '<br/><span class="customer_id">ID:'.$customer->ID.'</span>';						
					break;
				case 'customer_name' :
					the_title();
					$user_id = get_post_meta( $customer->ID, '_customer_user', true );
					if ( !empty($user_id) && $user_id != 0 ) {
						$username = get_post_meta( $customer->ID, '_customer_username', true );
						printf ('<br/><span class="customer_username">%s: %s</span>', __( 'username', 'wpo_wccm' ), $username);
					} else {
						echo '<br/><span class="customer_username guest">' . __( 'guest', 'wpo_wccm' ) . '</span>';						
					}
					do_action( 'wccm_after_customer_name_data', $customer, $user_id );
					break;	
				case 'customer_billing_address' :
					echo get_post_meta( $customer->ID, '_billing_address', true );
					printf('<br><a href="mailto:%1$s" class="icons email">%1$s</a>', get_post_meta( $customer->ID, '_email', true ) );
					printf('<br><a href="tel:%1$s" class="icons phone">%1$s</a>', get_post_meta( $customer->ID, '_billing_phone', true ) );
					break;
				case 'customer_shipping_address' :
					echo get_post_meta( $customer->ID, '_shipping_address', true );
					break;
				case 'customer_orders' :
					$email = get_post_meta( $customer->ID, '_email', true );

					if (empty($email)) {
						return;
					}
					
					$count = get_post_meta( $customer->ID, '_order_count', true );

					// if we have a user_id and WC2.3+, we can use wc_get_customer_total_spent
					$user_id = get_post_meta( $customer->ID, '_customer_user', true );
					if ( !empty($user_id) && $user_id != 0 && version_compare( WOOCOMMERCE_VERSION, '2.3', '>=' ) ) {
						$spent = wc_get_customer_total_spent( $user_id );
					} else {
						// queries for WC2.2+ (see http://develop.woothemes.com/woocommerce/2014/08/wc-2-2-order-statuses-plugin-compatibility/)
						if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '>=' ) ) {
							$spent = $wpdb->get_var( "SELECT SUM(meta2.meta_value)
								FROM $wpdb->posts as posts

								LEFT JOIN {$wpdb->postmeta} AS meta ON posts.ID = meta.post_id
								LEFT JOIN {$wpdb->postmeta} AS meta2 ON posts.ID = meta2.post_id

								WHERE   meta.meta_key       = '_billing_email'
								AND     meta.meta_value     = '$email'
								AND     posts.post_type     IN ('" . implode( "','", wc_get_order_types( 'reports' ) ) . "')
								AND     posts.post_status   IN ( 'wc-completed', 'wc-processing' )
								AND     meta2.meta_key      = '_order_total'
							" );
						} else { // backwards compatible queries for WC 2.1 and possibly 2.0
							$spent = $wpdb->get_var( "SELECT SUM(meta2.meta_value)
								FROM $wpdb->posts as posts

								LEFT JOIN {$wpdb->term_relationships} AS rel ON posts.ID=rel.object_ID
								LEFT JOIN {$wpdb->term_taxonomy} AS tax USING( term_taxonomy_id )
								LEFT JOIN {$wpdb->terms} AS term USING( term_id )
								LEFT JOIN {$wpdb->postmeta} AS meta ON posts.ID = meta.post_id
								LEFT JOIN {$wpdb->postmeta} AS meta2 ON posts.ID = meta2.post_id

								WHERE   meta.meta_key       = '_billing_email'
								AND     meta.meta_value     = '$email'
								AND     posts.post_type     = 'shop_order'
								AND     posts.post_status   = 'publish'
								AND     tax.taxonomy        = 'shop_order_status'
								AND     term.slug           IN ( '" . implode( "','", array( 'completed', 'processing' ) ) . "' )
								AND     meta2.meta_key      = '_order_total'
							" );
						}
					}

					$url = admin_url( 'edit.php?post_type=shop_order&s='.$email );
					$title = _n( 'order', 'orders', $count, 'wpo_wccm' );
					printf( '<a href="%s">%d %s</a><br/>%s', $url, $count, $title, wc_price( $spent ) );

					break;
				case 'customer_last_order' :
					$last_order = get_post_meta( $customer->ID, '_last_order', true );

					if ( !empty( $last_order ) && get_post_status( $last_order ) != 'trash' ) {
						if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '>=' ) ) {
							$order = wc_get_order( $last_order );
						} else {
							$order = new WC_Order( $last_order );
						}
						echo '<a href="' . admin_url( 'post.php?post=' . $order->id . '&action=edit' ) . '">' . $order->get_order_number() . '</a> &ndash; ' . date_i18n( get_option( 'date_format' ), strtotime( $order->order_date ) );
					} else echo '-';

					break;

			}
		}

		public function remove_submit_box () {
			remove_meta_box( 'submitdiv', 'wc_customers', 'side' );
			remove_meta_box( 'commentstatusdiv', 'wc_customers', 'side' );
			remove_meta_box( 'commentsdiv', 'wc_customers', 'side' );
		}

		public function add_meta_boxes() {
			$settings = get_option( 'wpo_wccm_settings' );

			add_meta_box(
				'wc_customers_address',
				__( 'Customer addresses', 'wpo_wccm' ),
				array( &$this, 'address_meta_box'),
				'wc_customers',
				'normal',
				'high'
			);
			add_meta_box(
				'wc_customers_orders',
				__( 'Customer orders', 'wpo_wccm' ),
				array( &$this, 'orders_meta_box'),
				'wc_customers',
				'normal',
				'high'
			);
			add_meta_box(
				'wc_customers_notes',
				__( 'Customer Notes', 'wpo_wccm' ),
				array( &$this, 'comments_meta_box'),
				'wc_customers',
				'normal',
				'high'
			);
			add_meta_box(
				'wc_customers_card',
				__( 'Customer Card', 'wpo_wccm' ),
				array( &$this, 'card_meta_box'),
				'wc_customers',
				'side',
				'high'
			);
			if (isset($settings['show_card'])) {
				add_meta_box(
					'wc_customers_card',
					__( 'Customer Card', 'wpo_wccm' ),
					array( &$this, 'card_meta_box'),
					'shop_order',
					'side',
					'default'
				);
			}


		}
		public function address_meta_box( $customer ) {
			global $wpo_wccm;
			$customer_number_raw = $wpo_wccm->functions->get_customer_number( $customer->ID, '', false );
			$customer_number = $wpo_wccm->functions->get_customer_number( $customer->ID, $customer_number_raw );
			$user_id = get_post_meta( $customer->ID, '_customer_user', true );

			$billing = get_post_meta( $customer->ID, '_billing_address', true );
			$shipping = get_post_meta( $customer->ID, '_shipping_address', true );
			if ($billing == $shipping) {
				$same_address = true;
			}

			?>
			<h2><?php the_title() ?></h2>
			<?php _e( 'Customer Number', 'wpo_wccm' ); ?>: <span class="customer-number"><input type="text" class="customer-number-input" name="_customer_number" value="<?php echo $customer_number_raw; ?>" data-original="<?php echo $customer_number_raw; ?>"><?php if ($customer_number !== $customer_number_raw) echo " ({$customer_number})"; ?></span> <a class="button save-customer-number"><?php _e( 'Save', 'wpo_wccm' ) ?></a>
			<table class="customer_addresses">
				<tr>
					<td><h4><?php _e( 'Billing Address', 'wpo_wccm' ); ?></h4></td>
					<td><h4><?php _e( 'Shipping Address', 'wpo_wccm' ); ?></h4></td>
				</tr>
				<tr>
					<td>
					<?php
					echo $billing;
					printf('<br><a href="mailto:%1$s" class="icons email">%1$s</a>', get_post_meta( $customer->ID, '_email', true ) );
					printf('<br><a href="tel:%1$s" class="icons phone">%1$s</a>', get_post_meta( $customer->ID, '_billing_phone', true ) );
					?>
					</td>
					<td class="<?php if (isset($same_address)) echo 'same'; ?>"><?php echo $shipping; ?></td>
				</tr>
			</table>
			<?php
			if (!empty($user_id) && $user_id != 0) {
				printf('<br><a href="user-edit.php?user_id=%1$s">%2$s</a>', absint( $user_id ), __( 'Edit user addresses', 'wpo_wccm' ) );
			} else {
				$last_order = get_post_meta( $customer->ID, '_last_order', true );
				printf('<br><a href="post.php?post=%1$s&action=edit">%2$s</a>', absint( $last_order ), __( 'Edit addresses in last order', 'wpo_wccm' ) );
			}
		}

		public function orders_meta_box( $customer ) {
			$customer_orders = get_post_meta( $customer->ID, '_customer_orders', true );
			// sort newest on top
			arsort($customer_orders);

			printf( __( 'Total orders: %s', 'wpo_wccm' ), count($customer_orders) );
			?>
			<table class="customer_orders">

				<thead>
					<tr>
						<th class="order-number"><span class="nobr"><?php _e( 'Order', 'woocommerce' ); ?></span></th>
						<th class="order-date"><span class="nobr"><?php _e( 'Date', 'woocommerce' ); ?></span></th>
						<th class="order-status"><span class="nobr"><?php _e( 'Status', 'woocommerce' ); ?></span></th>
						<th class="order-purchase"><span class="nobr"><?php _e( 'Purchased', 'woocommerce' ); ?></span></th>
						<th class="order-total"><span class="nobr"><?php _e( 'Total', 'woocommerce' ); ?></span></th>
					</tr>
				</thead>

				<tbody><?php
					foreach ( $customer_orders as $customer_order ) {
						if ( !get_post_status ( $customer_order ) ) {
							continue;
						}
						if ( version_compare( WOOCOMMERCE_VERSION, '2.2', '>=' ) ) {
							$order = wc_get_order( $customer_order );
							$status = wc_get_order_status_name( $order->get_status() );
						} else {
							$order = new WC_Order( $customer_order );
							$status = $order->status;
						}

						// $order->populate( $customer_order );
						$item_count = $order->get_item_count();
						$order_url = admin_url( 'post.php?action=edit&post='.$order->id );
						?><tr class="order">
							<td class="order-number">
								<?php
								if ($status == 'trash') {
									echo $order->get_order_number();
								} else {
									printf('<a href="%s">%s</a>', $order_url, $order->get_order_number());
								}
								?>
							</td>
							<td class="order-date">
								<time datetime="<?php echo date( 'Y-m-d', strtotime( $order->order_date ) ); ?>" title="<?php echo esc_attr( strtotime( $order->order_date ) ); ?>"><?php echo date_i18n( get_option( 'date_format' ), strtotime( $order->order_date ) ); ?></time>
							</td>
							<td class="order-status" style="text-align:left; white-space:nowrap;">
								<?php echo $status; ?>
							</td>
							<td class="order-purchased">
								<?php printf( _n( '%s item', '%s items', $item_count, 'woocommerce' ), $item_count ); ?>
							</td>
							<td class="order-total">
								<?php echo $order->get_formatted_order_total(); ?>
							</td>
					</tr><?php
					}
				?></tbody>

			</table>
			<?php

			$email = get_post_meta( $customer->ID, '_email', true );
			$url = admin_url( 'edit.php?post_type=shop_order&s='.$email );
			$title = __( 'View/edit orders in WooCommerce', 'wpo_wccm' );
			printf( '<a href="%s">%s &rarr;</a>', $url, $title );
		}

		public function comments_meta_box( $customer ) {
			global $post;

			$args = array(
				'post_id'   => $post->ID,
				'approve'   => 'approve',
				'type'      => 'customer_note'
			);

			$notes = get_comments( $args );

			echo '<ul class="customer_notes">';

			if ( $notes ) {

				foreach( $notes as $note ) {
					?>
					<li rel="<?php echo absint( $note->comment_ID ) ; ?>" class="note">
						<div class="note_content">
							<?php echo wpautop( wptexturize( wp_kses_post( $note->comment_content ) ) ); ?>
						</div>
						<p class="meta">
							<abbr class="exact-date" title="<?php echo $note->comment_date_gmt; ?> GMT">
								<?php printf( __( 'added %s ago', 'wpo_wccm' ), human_time_diff( strtotime( $note->comment_date_gmt ), current_time( 'timestamp', 1 ) ) ); ?>
							</abbr>
							<?php printf( ' ' . __( 'by %s', 'wpo_wccm' ), $note->comment_author ); ?>
							<a href="#" class="delete_note"><?php _e( 'Delete note', 'wpo_wccm' ); ?></a>
						</p>
					</li>
					<?php
				}

			} else {
				echo '<li>' . __( 'There are no notes for this customer yet.', 'wpo_wccm' ) . '</li>';
			}

			echo '</ul>';
			?>
			<div class="add_customer_note">
				<h4><?php _e( 'Add note', 'wpo_wccm' ); ?></h4>
				<p>
					<textarea type="text" name="customer_note" id="add_customer_note" class="input-text" rows="5" style="width:100%"></textarea>
				</p>
				<p>

				<a href="#" class="add_customer_note button"><?php _e( 'Add', 'wpo_wccm' ); ?></a>
				</p>
			</div>
			<?php
		}

		public function card_meta_box( $post ) {
			global $post_type, $pagenow;
			if ( $post_type == 'shop_order' ) {
				$customer_id = get_post_meta( $post->ID, '_customer_id', true );
			} else {
				// post_type = customer
				$customer_id = $post->ID;
			}

			$settings = get_option( 'wpo_wccm_settings' );

			// remove incomplete/empty fields
			$custom_fields = isset( $settings['custom_fields'] ) ? $settings['custom_fields'] : array();
			foreach ( $custom_fields as $key => $custom_field ) {
				if ( empty( $custom_field['label']) || empty($custom_field['field']) ) {
					unset( $custom_fields[$key] );
				}
			}

			if ( !isset($custom_fields) || empty($custom_fields) ) {
				$message = __( 'No fields defined for the customer card!', 'wpo_wccm');
				$settings_url = admin_url( 'admin.php?page=wpo_wccm_options_page' );
				$link = __( 'Configure fields', 'wpo_wccm');
				printf( '<p>%s</p><a href="%s">%s</a>', $message, $settings_url, $link );
			} else {
				$new_post = in_array( $pagenow, array( 'post-new.php' ) );
				if ( $new_post ) {
					echo '<fieldset class="wccm_disabled" disabled="disabled">';
				}

				foreach ( $custom_fields as $custom_field ) {
					$type		= $custom_field['type'];
					$label		= $custom_field['label'];
					$dbname		= '_'.$custom_field['field'];
					$name		= 'wpo_wccm'.$dbname;
					$current	= get_post_meta( $customer_id, $dbname, true );
					// print_r($custom_field);
					switch ($custom_field['type']) {
						case 'checkbox':
							$value		= 1;
							$checked	= checked( $value, $current, false );
							printf('<p><input type="checkbox" id="%1$s" name="%1$s" value="%2$s"%3$s /> %4$s</p>', $name, $value, $checked, $label);
							break;
						
						default:
							printf('<p>%3$s:<br/><input type="text" id="%1$s" name="%1$s" value="%2$s"/></p>', $name, $current, $label);
							break;
					}
				}

				?>
				<input type="submit" class="button wccm_save_customer_card" name="save" value="<?php _e( 'Save customer card', 'wpo_wccm' ); ?>"/>
				<?php
				if ( $new_post ) {
					echo '</fieldset>';
				}
			}

			if ( $post_type == 'shop_order' ) {
				$url = admin_url( 'post.php?action=edit&post='.$customer_id );
				$title = __( 'View customer details', 'wpo_wccm' );
				printf( '<p><a href="%s">%s &rarr;</a></p>', $url, $title );
			}
		}

		public function add_order_actions ( $order ) {
			$customer_id = get_post_meta( $order->id, '_customer_id', true );
			if ( !empty( $customer_id ) ) {
				$url = admin_url( 'post.php?action=edit&post='.$customer_id );
				$alt = __( 'View customer', 'wpo_wccm' );
				printf('<a href="%1$s" class="button tips wpo_wccm_customer_action" target="_blank" alt="%2$s" data-tip="%2$s" style="height: 26px; padding: 2px;"><span class="dashicons dashicons-admin-users"></span></a>', $url, $alt );
			}
		}

		/**
		 * Change messages when a post type is updated.
		 *
		 * @param  array $messages
		 * @return array
		 */
		public function post_updated_messages( $messages ) {
			global $post, $post_ID;

			$messages['wc_customers'] = array(
				0 => '', // Unused. Messages start at index 1.
				1 => __( 'Customer updated.', 'wpo_wccm' ),
				2 => __( 'Custom field updated.', 'wpo_wccm' ),
				3 => __( 'Custom field deleted.', 'wpo_wccm' ),
				4 => __( 'Customer updated.', 'wpo_wccm' ),
				5 => isset($_GET['revision']) ? sprintf( __( 'Customer restored to revision from %s', 'wpo_wccm' ), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false,
				6 => __( 'Customer updated.', 'wpo_wccm' ),
				7 => __( 'Customer saved.', 'wpo_wccm' ),
				8 => __( 'Customer submitted.', 'wpo_wccm' ),
				9 => sprintf( __( 'Customer scheduled for: <strong>%1$s</strong>.', 'wpo_wccm' ),
				  date_i18n( __( 'M j, Y @ G:i', 'wpo_wccm' ), strtotime( $post->post_date ) ) ),
				10 => __( 'Customer draft updated.', 'wpo_wccm' )
			);

			return $messages;
		}

		public function filter_by_customer_note ( $types ) {
			$types['customer_note'] = __( 'Customer Notes', 'wpo_wccm' );
			return $types;
		}

		/**
		 * Display Customer number in emails
		 *
		 * @param array $fields
		 * @param bool $sent_to_admin (default: false)
		 * @param mixed $order
		 * @return array $fields
		 */
		public function email_customer_number ( $fields, $sent_to_admin, $order ) {
			global $wpo_wccm;
			if ( !empty($this->settings['number_wc_emails']) && !empty($order->customer_number) ) {
				$customer_number = $wpo_wccm->functions->get_customer_number( $order->id );
				$fields['customer_number'] = array(
					'label' => __( 'Customer number', 'wpo_wccm' ),
					'value' => wptexturize( $customer_number )
				);
			}
			return $fields;
		}

		public function my_account_customer_number () {
			global $wpo_wccm;
			$user_id = get_current_user_id();
			$customer_number = $wpo_wccm->functions->get_customer_number( $user_id );
			if ( !empty($this->settings['number_my_account']) && !empty($customer_number) ) {
				?>
				<header class="title">
					<h3><?php _e( 'Customer number', 'wpo_wccm' ); ?></h3>
				</header>
				<?php echo $customer_number; ?>
				<?php
			}
		}

		/**
		 * Show Address Fields on edit user pages.
		 *
		 * @param WP_User $user
		 */
		public function user_profile_customer_number( $user ) {
			global $wpo_wccm;
			$customer_number = $wpo_wccm->functions->get_customer_number( $user->ID );
			if ( !empty($customer_number) ) {
				?>
				<!-- <h3><?php _e( 'Customer number', 'wpo_wccm' ); ?></h3> -->
				<table class="form-table">
					<tr>
						<th><?php _e( 'Customer number', 'wpo_wccm' ); ?></th>
						<td>
							<span class="customer_number"><?php echo $customer_number; ?></span>
						</td>
					</tr>
				</table>
				<?php
			}
		}

	} // end class
} // end class_exists